@@ -34,6 +34,7 @@ impl MapArrayDecoder {
     pub fn new(
         data_type: DataType,
         coerce_primitive: bool,
+        strict_mode: bool,
         is_nullable: bool,
     ) -> Result<Self, ArrowError> {
         let fields = match &data_type {
@@ -56,11 +57,13 @@ impl MapArrayDecoder {
         let keys = make_decoder(
             fields[0].data_type().clone(),
             coerce_primitive,
+            strict_mode,
             fields[0].is_nullable(),
         )?;
         let values = make_decoder(
             fields[1].data_type().clone(),
             coerce_primitive,
+            strict_mode,
             fields[1].is_nullable(),
         )?;
 
